/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.transformers;

import com.google.common.base.Predicate;
import com.google.common.collect.Iterators;
import net.minecraft.block.Block;
import net.minecraft.block.BlockDynamicLiquid;
import net.minecraft.block.BlockStaticLiquid;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fluids.BlockFluidBase;
import prospector.shootingstar.ShootingStar;
import prospector.shootingstar.model.ModelCompound;
import reborncore.common.BaseTileBlock;
import techreborn.client.TechRebornCreativeTab;
import techreborn.lib.ModInfo;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/**
 * Created by Rushmead
 */
public abstract class BlockTransformer extends BaseTileBlock {
	public static PropertyDirection FACING = PropertyDirection.func_177712_a("facing", Facings.ALL);
	public String name;

	public BlockTransformer(String name) {
		super(Material.field_151573_f);
		func_149711_c(2f);
		func_149647_a(TechRebornCreativeTab.instance);
		this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(FACING, EnumFacing.NORTH));
		this.name = name;
		ShootingStar.registerModel(new ModelCompound(ModInfo.MOD_ID, this, "machines/energy"));
	}

	protected BlockStateContainer func_180661_e() {
		FACING = PropertyDirection.func_177712_a("facing", Facings.ALL);
		return new BlockStateContainer(this, FACING);
	}

	@Override
	public void func_176213_c(World worldIn, BlockPos pos, IBlockState state) {
		super.func_176213_c(worldIn, pos, state);
		this.setDefaultFacing(worldIn, pos, state);
	}

	private void setDefaultFacing(World worldIn, BlockPos pos, IBlockState state) {
		if (!worldIn.field_72995_K) {
			IBlockState sate = worldIn.func_180495_p(pos.func_177978_c());
			Block block = sate.func_177230_c();
			IBlockState state1 = worldIn.func_180495_p(pos.func_177968_d());
			Block block1 = state1.func_177230_c();
			IBlockState state2 = worldIn.func_180495_p(pos.func_177976_e());
			Block block2 = state2.func_177230_c();
			IBlockState state3 = worldIn.func_180495_p(pos.func_177974_f());
			Block block3 = state3.func_177230_c();
			EnumFacing enumfacing = state.func_177229_b(FACING);

			if (enumfacing == EnumFacing.NORTH && block.func_149730_j(state) && !block1.func_149730_j(state1)) {
				enumfacing = EnumFacing.SOUTH;
			} else if (enumfacing == EnumFacing.SOUTH && block1.func_149730_j(state1) && !block.func_149730_j(state)) {
				enumfacing = EnumFacing.NORTH;
			} else if (enumfacing == EnumFacing.WEST && block2.func_149730_j(state2) && !block3.func_149730_j(state2)) {
				enumfacing = EnumFacing.EAST;
			} else if (enumfacing == EnumFacing.EAST && block3.func_149730_j(state3) && !block2.func_149730_j(state2)) {
				enumfacing = EnumFacing.WEST;
			}

			worldIn.func_180501_a(pos, state.func_177226_a(FACING, enumfacing), 2);
		}
	}

	@Override
	public void func_180633_a(World worldIn, BlockPos pos, IBlockState state, EntityLivingBase placer,
	                            ItemStack stack) {
		super.func_180633_a(worldIn, pos, state, placer, stack);
		EnumFacing facing = placer.func_174811_aO().func_176734_d();
		if (placer.field_70125_A < -50) {
			facing = EnumFacing.DOWN;
		} else if (placer.field_70125_A > 50) {
			facing = EnumFacing.UP;
		}
		setFacing(facing, worldIn, pos);
	}

	protected List<ItemStack> dropInventory(IBlockAccess world, BlockPos pos, ItemStack itemToDrop) {
		TileEntity tileEntity = world.func_175625_s(pos);

		if (tileEntity == null) {
			System.out.print("Null");
			return null;
		}
		if (!(tileEntity instanceof IInventory)) {

			System.out.print("Not INstance");
			return null;
		}

		IInventory inventory = (IInventory) tileEntity;

		List<ItemStack> items = new ArrayList<>();

		for (int i = 0; i < inventory.func_70302_i_(); i++) {
			ItemStack itemStack = inventory.func_70301_a(i);

			if (itemStack == ItemStack.field_190927_a) {
				continue;
			}
			if (itemStack != ItemStack.field_190927_a && itemStack.func_190916_E() > 0) {
				if (itemStack.func_77973_b() instanceof ItemBlock) {
					if (((ItemBlock) itemStack.func_77973_b()).func_179223_d() instanceof BlockFluidBase
						|| ((ItemBlock) itemStack.func_77973_b()).func_179223_d() instanceof BlockStaticLiquid
						|| ((ItemBlock) itemStack.func_77973_b()).func_179223_d() instanceof BlockDynamicLiquid) {
						continue;
					}
				}
			}
			items.add(itemStack.func_77946_l());
		}
		items.add(itemToDrop.func_77946_l());
		return items;

	}

	@Override
	public int func_176201_c(IBlockState state) {
		int facingInt = getSideFromEnum(state.func_177229_b(FACING));
		return facingInt;
	}

	@Override
	public IBlockState func_176203_a(int meta) {
		boolean active = false;
		EnumFacing facing = getSideFromint(meta);
		return this.func_176223_P().func_177226_a(FACING, facing);
	}

	public void setFacing(EnumFacing facing, World world, BlockPos pos) {
		world.func_175656_a(pos, world.func_180495_p(pos).func_177226_a(FACING, facing));
	}

	public EnumFacing getSideFromint(int i) {
		if (i == 0) {
			return EnumFacing.NORTH;
		} else if (i == 1) {
			return EnumFacing.SOUTH;
		} else if (i == 2) {
			return EnumFacing.EAST;
		} else if (i == 3) {
			return EnumFacing.WEST;
		} else if (i == 4) {
			return EnumFacing.UP;
		} else if (i == 5) {
			return EnumFacing.DOWN;
		}
		return EnumFacing.NORTH;
	}

	public int getSideFromEnum(EnumFacing facing) {
		if (facing == EnumFacing.NORTH) {
			return 0;
		} else if (facing == EnumFacing.SOUTH) {
			return 1;
		} else if (facing == EnumFacing.EAST) {
			return 2;
		} else if (facing == EnumFacing.WEST) {
			return 3;
		} else if (facing == EnumFacing.UP) {
			return 4;
		} else if (facing == EnumFacing.DOWN) {
			return 5;
		}
		return 0;
	}

	public EnumFacing getFacing(IBlockState state) {
		return state.func_177229_b(FACING);
	}

	public enum Facings implements Predicate<EnumFacing>, Iterable<EnumFacing> {
		ALL;

		public EnumFacing[] facings() {
			return new EnumFacing[] { EnumFacing.NORTH, EnumFacing.EAST, EnumFacing.SOUTH, EnumFacing.WEST,
				EnumFacing.UP, EnumFacing.DOWN };
		}

		public EnumFacing random(Random rand) {
			EnumFacing[] aenumfacing = this.facings();
			return aenumfacing[rand.nextInt(aenumfacing.length)];
		}

		public boolean apply(EnumFacing p_apply_1_) {
			return p_apply_1_ != null;
		}

		public Iterator<EnumFacing> iterator() {
			return Iterators.forArray(this.facings());
		}
	}

}
